precision highp float;
#if __VERSION__ >= 300
    layout (location = 0) out vec4 FragColor;
    in vec2 varyTexCoord;
#else
    varying vec2 varyTexCoord;
#endif

uniform sampler2D texture_y;
uniform sampler2D texture_u;
uniform sampler2D texture_v;
uniform float stride_scale_y;
uniform float stride_scale_u;
uniform float stride_scale_v;

uniform mat4 color_space;

uniform int rotate;
uniform vec4 crop; // left right top bottom
uniform float transparent;

void main(){
    vec2 coord = varyTexCoord;
    float cropWidth = crop.y - crop.x; // 处理截取
    float cropHeight = crop.w - crop.z;
    coord = vec2(coord.x * cropWidth / 1.0 + crop.x, coord.y * cropHeight / 1.0 + crop.z);
    if (rotate == 270){ // 处理旋转
       coord = vec2(1.0 - coord.y, coord.x);
    } else if (rotate == 180) {
       coord = vec2(1.0 - coord.x, 1.0 - coord.y);
    } else if (rotate == 90) {
       coord = vec2(coord.y, 1.0 - coord.x);
    }
#if __VERSION__ >= 300
    float y = texture(texture_y, vec2(coord.x * stride_scale_y, coord.y)).a;
    float u = texture(texture_u, vec2(coord.x * stride_scale_u, coord.y)).a;
    float v = texture(texture_v, vec2(coord.x * stride_scale_v, coord.y)).a;
    vec4 yuv = vec4(y, u, v, 1.0);
    vec4 rgba = color_space * yuv;
    rgba.a = 1.0 - transparent;
    FragColor = rgba;
#else
    float y = texture2D(texture_y, vec2(coord.x * stride_scale_y, coord.y)).a;
    float u = texture2D(texture_u, vec2(coord.x * stride_scale_u, coord.y)).a;
    float v = texture2D(texture_v, vec2(coord.x * stride_scale_v, coord.y)).a;
    vec4 yuv = vec4(y, u, v, 1.0);
    vec4 rgba = color_space * yuv;
    rgba.a = 1.0 - transparent;
    gl_FragColor = rgba;
#endif
}
