precision highp float;
#if __VERSION__ >= 300
    layout (location = 0) out vec4 FragColor;
    in vec2 varyTexCoord;
#else
    varying vec2 varyTexCoord;
#endif

uniform sampler2D sampleTexture;
uniform float strideScale;
uniform int rotate;
uniform float transparent;
uniform vec4 crop; // left right top bottom
uniform mat4 rgbaMat;

void main(){
   vec2 coord = varyTexCoord;
   float cropWidth = crop.y - crop.x; // 处理截取
   float cropHeight = crop.w - crop.z;
   coord = vec2(coord.x * cropWidth / 1.0 + crop.x, coord.y * cropHeight / 1.0 + crop.z);
   if (rotate == 270){ // 处理旋转
      coord = vec2(1.0 - coord.y, coord.x);
   } else if (rotate == 180) {
      coord = vec2(1.0 - coord.x, 1.0 - coord.y);
   } else if (rotate == 90) {
      coord = vec2(coord.y, 1.0 - coord.x);
   }
   coord = vec2(coord.x * strideScale, coord.y);
#if __VERSION__ >= 300
   vec4 color = texture(sampleTexture, coord);
   color = transpose(rgbaMat) * color;
   FragColor = vec4(color.r, color.g, color.b, (1.0 - transparent) * color.a);
#else
   vec4 color = texture2D(sampleTexture, coord);
   color = transpose(rgbaMat) * color;
   gl_FragColor = vec4(color.r, color.g, color.b, (1.0 - transparent) * color.a);
#endif
}