// 横向开幕转场
precision highp float;
#if __VERSION__ >= 300
    layout (location = 0) out vec4 FragColor;
    in vec2 varyCoord;
#else
    varying vec2 varyCoord;
#endif

uniform vec2 sceneSize;
uniform vec2 lastCenter;
uniform vec2 lastSize;
uniform mat4 lastAngleMat;
uniform vec2 nextCenter;
uniform vec2 nextSize;
uniform mat4 nextAngleMat;
uniform sampler2D last_texture_y_or_bgra;
uniform sampler2D last_texture_u;
uniform sampler2D last_texture_v;
uniform sampler2D next_texture_y_or_bgra;
uniform sampler2D next_texture_u;
uniform sampler2D next_texture_v;
uniform float last_stride_scale_y_or_bgra;
uniform float last_stride_scale_u;
uniform float last_stride_scale_v;
uniform float next_stride_scale_y_or_bgra;
uniform float next_stride_scale_u;
uniform float next_stride_scale_v;
uniform vec4 lastSampleCrop;
uniform vec4 nextSampleCrop;
uniform int lastSampleVerticalMirror;
uniform int nextSampleVerticalMirror;
uniform int lastSampleHorizontalMirror;
uniform int nextSampleHorizontalMirror;
uniform int lastSampleRotate;
uniform int nextSampleRotate;
uniform float dividing;

uniform int last_is_bgra;
uniform int next_is_bgra;

uniform mat4 last_color_space;
uniform mat4 next_color_space;

vec2 transCoord(vec2 pos, vec2 center, mat4 mat) {
  vec2 ret = pos - center;
  ret = (mat * vec4(ret, 0.0, 1.0)).xy;
  return ret;
}

vec2 tranlateTexCoord(vec2 coord, vec4 crop, int verticalMirror, int horizontalMirror, int sampleRotate){
    float cropWidth = crop.y - crop.x; // 处理截取
    float cropHeight = crop.w - crop.z;
    coord = vec2(coord.x * cropWidth / 1.0 + crop.x, coord.y * cropHeight / 1.0 + crop.z);
    if (verticalMirror != 0)
        coord = vec2(coord.x, 1.0 - coord.y);
    if (horizontalMirror != 0)
        coord = vec2(1.0 - coord.x, coord.y);
    if (sampleRotate == 270){
       coord = vec2(1.0 - coord.y, coord.x);
    } else if (sampleRotate == 180) {
       coord = vec2(1.0 - coord.x, 1.0 - coord.y);
    } else if (sampleRotate == 90) {
       coord = vec2(coord.y, 1.0 - coord.x);
    }
    return coord;
}

vec4 SampleLast(vec2 coord) {
    vec4 rgba;
    if (last_is_bgra != 0) {
#if __VERSION__ >= 300
        rgba = texture(last_texture_y_or_bgra, vec2(coord.x * last_stride_scale_y_or_bgra, coord.y));
#else
        rgba = texture2D(last_texture_y_or_bgra, vec2(coord.x * last_stride_scale_y_or_bgra, coord.y));
#endif
        rgba = vec4(rgba.b, rgba.g, rgba.r, rgba.a);
    } else {
        vec4 yuv;
#if __VERSION__ >= 300
        float y = texture(last_texture_y_or_bgra, vec2(coord.x * last_stride_scale_y_or_bgra, coord.y)).a;
        float u = texture(last_texture_u, vec2(coord.x * last_stride_scale_u, coord.y)).a;
        float v = texture(last_texture_v, vec2(coord.x * last_stride_scale_v, coord.y)).a;
        yuv = vec4(y, u, v, 1.0);
#else
        float y = texture2D(last_texture_y_or_bgra, vec2(coord.x * last_stride_scale_y_or_bgra, coord.y)).a;
        float u = texture2D(last_texture_u, vec2(coord.x * last_stride_scale_u, coord.y)).a;
        float v = texture2D(last_texture_v, vec2(coord.x * last_stride_scale_v, coord.y)).a;
        yuv = vec4(y, u, v, 1.0);
#endif
        rgba = last_color_space * yuv;
        rgba.a = 1.0;
    }
    return rgba;
}

vec4 SampleNext(vec2 coord) {
    vec4 rgba;
    if (next_is_bgra != 0) {
#if __VERSION__ >= 300
        rgba = texture(next_texture_y_or_bgra, vec2(coord.x * next_stride_scale_y_or_bgra, coord.y));
#else
        rgba = texture2D(next_texture_y_or_bgra, vec2(coord.x * next_stride_scale_y_or_bgra, coord.y));
#endif
        rgba = vec4(rgba.b, rgba.g, rgba.r, rgba.a);
    } else {
        vec4 yuv;
#if __VERSION__ >= 300
        float y = texture(next_texture_y_or_bgra, vec2(coord.x * next_stride_scale_y_or_bgra, coord.y)).a;
        float u = texture(next_texture_u, vec2(coord.x * next_stride_scale_u, coord.y)).a;
        float v = texture(next_texture_v, vec2(coord.x * next_stride_scale_v, coord.y)).a;
        yuv = vec4(y, u, v, 1.0);
#else
        float y = texture2D(next_texture_y_or_bgra, vec2(coord.x * next_stride_scale_y_or_bgra, coord.y)).a;
        float u = texture2D(next_texture_u, vec2(coord.x * next_stride_scale_u, coord.y)).a;
        float v = texture2D(next_texture_v, vec2(coord.x * next_stride_scale_v, coord.y)).a;
        yuv = vec4(y, u, v, 1.0);
#endif
        rgba = next_color_space * yuv;
        rgba.a = 1.0;
    }
    return rgba;
}

void main() {
    vec4 color;
    float left = - dividing;
    float right = dividing;
    vec2 coord = vec2(varyCoord.x, -varyCoord.y); // 将Y轴坐标与场景坐标方向同步
    vec2 scenePos = vec2(coord.x / 2.0 * sceneSize.x, coord.y / 2.0 * sceneSize.y);
    vec2 nextPos = transCoord(scenePos, nextCenter, nextAngleMat);
    if (abs(nextPos.x) <= nextSize.x / 2.0 && abs(nextPos.y) <= nextSize.y / 2.0) {
        vec2 texCoord = vec2(nextPos.x / nextSize.x + 0.5, nextPos.y / nextSize.y + 0.5);
        texCoord = tranlateTexCoord(texCoord, nextSampleCrop, nextSampleVerticalMirror, nextSampleHorizontalMirror, nextSampleRotate);

        color = SampleNext(texCoord);
    } else {
        color = vec4(0.0, 0.0, 0.0, 0.0);
    }
    if (coord.x <= left || coord.x >= right) {
        vec2 lastPos;
        if (coord.x <= left) {
            lastPos = vec2(coord.x - left, coord.y);
        } else {
            lastPos = vec2(coord.x - right, coord.y);
        }
        scenePos = vec2(lastPos.x / 2.0 * sceneSize.x, lastPos.y / 2.0 * sceneSize.y);
        lastPos = transCoord(scenePos, lastCenter, lastAngleMat);
        if (abs(lastPos.x) <= lastSize.x / 2.0 && abs(lastPos.y) <= lastSize.y / 2.0) {
            vec2 texCoord = vec2(lastPos.x / lastSize.x + 0.5, lastPos.y / lastSize.y + 0.5);
            texCoord = tranlateTexCoord(texCoord, lastSampleCrop, lastSampleVerticalMirror, lastSampleHorizontalMirror, lastSampleRotate);

            color = SampleLast(texCoord);
        }
    }
#if __VERSION__ >= 300
    FragColor = color;
#else
    gl_FragColor = color;
#endif
}